#define F_CPU 16000000UL
#include <avr/io.h>
#include <util/delay.h>
#include <stdlib.h>

int water_level = 0;
int humidity = 0;

const uint8_t PIN_RED = PB3;    // Pin 11
const uint8_t PIN_GREEN = PB2;  // Pin 10
const uint8_t PIN_BLUE = PB1;   // Pin 9
const uint8_t PIN_BUZZER = PD7; // Pin 7

void USART_Init(unsigned int baud_rate)
{
  UBRR0 = 8;
 
  /* porneste transmitatorul */
  UCSR0B = (1 << TXEN0) | (1 << RXEN0);
 
  /* seteaza formatul frame-ului: 8 biti de date, 1 bit de stop, fara paritate */
  UCSR0C &= ~(1 << USBS0);
  UCSR0C |= (3 << UCSZ00);
}
void USART_Transmit(unsigned char data)
{
  while (!(UCSR0A & (1 << UDRE0)));
  UDR0 = data;
}

void setup()
{
  USART_Init(115200);
  DDRC |= (1 << PC5);
  DDRD |= (1 << PD7);

  DDRB |= (1 << PIN_RED) | (1 << PIN_GREEN) | (1 << PIN_BLUE);  // Set RGB LED pins as output

  // Initializam ADCC
  ADMUX = (0 << REFS1) | (1 << REFS0);  // AVcc referinta
  ADMUX |= (0 << ADLAR);                // right-adjust
  ADCSRA = (1 << ADEN) | (1 << ADPS2) | (1 << ADPS1) | (1 << ADPS0);  // activeaza prescaler 128
  
  // timer1 pt PWM PIN 11 (RED)
  TCCR1A |= (1 << COM1A1) | (1 << WGM10);  // seteaza mod PWM non-inverted, activeaza OC1A (Pin 11))
  TCCR1B |= (1 << WGM12) | (1 << CS10);    // seteaza mod PWM fast, fara prescaler
  DDRB |= (1 << PIN_RED);                  // seteaza Pin 11 ca output
  
  // timer1 pt PWM PIN 10 (GREEN)
  TCCR2A |= (1 << COM2A1) | (1 << WGM20);  // seteaza mod PWM non-inverted, activeaza OC2A (Pin 10)
  TCCR2B |= (1 << CS20);                                 // fara prescaler
  DDRB |= (1 << PIN_GREEN);                               // seteaza Pin 10 ca output
  
  // timer0 pt PWM PIN 9(BLUE)
  TCCR0A |= (1 << COM0A1) | (1 << WGM01) | (1 << WGM00);  // seteaza mod PWM non-inverted, activeaza OC0A (Pin 9)
  TCCR0B |= (1 << CS00);                                 // fara prescaler
  DDRB |= (1 << PIN_BLUE);                               // seteaza Pin 9 ca output

  PORTB |= (1 << PIN_RED);
  PORTB |= (1 << PIN_GREEN);
  PORTB |= (1 << PIN_BLUE);

  TCCR1B |= (1 << WGM12) | (1 << CS12);    // seteaza mod CTC, prescaler de 256
}

void loop()
{
  delay(10);
  
  // porneste ADC pentru masurarea nivelului apei
  ADMUX = (0 << REFS1) | (1 << REFS0) | (0 << MUX3) | (0 << MUX2) | (0 << MUX1) | (0 << MUX0);  // AVcc referinta, selecteaza ADC0 (A0 pin)
  ADCSRA |= (1 << ADSC);
  
  // trebuie sa se termine citirea
  while (ADCSRA & (1 << ADSC));
  
  water_level = ADC;

  // porneste ADC pentru masurarea umiditatii
  ADMUX = (0 << REFS1) | (1 << REFS0) | (0 << MUX3) | (0 << MUX2) | (0 << MUX1) | (1 << MUX0);  // AVcc referinta, selecteaza ADC1 (A1 pin)
  ADCSRA |= (1 << ADSC);
  
  // trebuie sa se termine citirea
  while (ADCSRA & (1 << ADSC));

  humidity = ADC;
  delay(1000);
  char buffer[5];
  itoa(humidity, buffer, 10);
  //transmitem un string catre esp
  for (int i = 0; i < strlen(buffer); i++) {
    USART_Transmit(buffer[i]);
  }

  if (humidity > 600 && water_level > 80)
  {
    PORTC |= (1 << PC5);  // porneste pompa(A5 pin)
  }
  else
  {
    PORTC &= ~(1 << PC5);  // opreste pompa (A5 pin)
  }

  if (water_level < 100)
  {
    // avertisment nivel apa
    tone(PD7, 1000);
  }
  else
  {
     noTone(PD7);
  }

  if (humidity > 900)
  {
    // sol uscat, culoare rosie
    OCR1A = 255;
    OCR2A = 255;
    PORTB |= (1 << PIN_RED);
    PORTB &= ~(1 << PIN_GREEN);
    PORTB &= ~(1 << PIN_BLUE);
  }
  else if (humidity > 600 && humidity < 900)
  {
    // sol mediu, culoare galbena
    OCR1A = 255;
    OCR2A = 80;
    PORTB |= (1 << PIN_RED);
    PORTB |= (1 << PIN_GREEN);
    PORTB &= ~(1 << PIN_BLUE);
  }
  else if (humidity < 600)
  {
    // sol ud, culoare verde
    PORTB &= ~(1 << PIN_RED);
    OCR2A = 255;
    PORTB |= (1 << PIN_GREEN);
    PORTB &= ~(1 << PIN_BLUE);
  }
}
